%{
##########################################################################
##########################################################################
Peter Maxted

Code for Simple Buffer Stock Model

Model Outline:
Let Y_t denote the income earned by the agent at the start of period t.
        --> Case 1. Deterministic y = 1 during work; y = 0 during retirement

        --> Case 2. E[y] = 1 during work with f(y); y = 0 during retirement
                        f(y) = uniform[0.5, 1.5]

Let X_t represent liquid assets in period t (after earning income Y_t)
        -->X_t has a minimum (credit limit) set by variable borrowingLimit
        -->X_t earns a real rate of return R

Let I_t denote investment into the liquid asset X during period t

Dynamic Budget Constraint: X_t+1 = R(X_t - C_t) + Y_t+1
Borrowing Constraint:  X_t+1 - Y_t+1 = R(X_t - C_t) >= credit limit
    NOTE to 2010C students: my "credit limit" is different than the "Borrowing Limit"
        defined in 2010C Lecture. The "credit limit" in this code equals -R*L of
        "Borrowing Limit" L defined in class. 

------------------------------------------------------------------
State Variables: X_t, t

Choice Variable: I_t
        -->Consumption is calculated as a residual
------------------------------------------------------------------


Agent Decision Framework:
Agent's have log utility and discount at rate delta

Let V(X_t+1) denote the continuation payoff of self t (given state variables X_t+1, t+1)
	--> Self t's Bellman equation is: max[ u(C_t) + delta * E(V(X_t+1)) ]

UPDATE 1/8/17: Now allowing for sophisticated and naive quasi-hyperbolic discounting.
	--> If specified, self t's Bellman equation is: max[ u(C_t) + beta-hat* delta * E(V(X_t+1)) ]


Simulation:
The code first generates policy and value functions at every combination of state variables.
	--> See function backwardInduction and code backwardInduction_general.m 
	--> E.g., variables C__, V__

Using these policy and value functions, profiles for 5000 simulated agents are generated 
	--> See function simulateDecisions and code simulateDecisions.m
	--> E.g., variables realC__, realY__

##########################################################################
##########################################################################
%}

clear all
addpath('main')
addpath('check') %Includes a small code that checks cases 1-3 with analytic solutions
addpath('graphs_for_lecture') %Links to a code that generates graphs used in lecture

%%-------------------------------------------------------------------------
%%-------------------------------------------------------------------------
%{
%%Pre-specified cases (Steps toward a simple buffer stock model)
    1. [NS] Deterministic y = 1 during work and y = 0 during infinite
    retirement. R = 1.05. delta*R = 1. Log utility. No liquidity constraint
    
    2. [NS] Case 1, but now delta = 0.9
    
    3. [S] Case 2, but infinite working life and no retirement

    4. [S] Case 3, but with stochastic income f(y) = uniform[0.5, 1.5]

    5. [NS] Case 4, but with finite lifetime

    6. [NS] Case 5, but with a liquidity constraint R(x_t - c_t) >= -0.5
	----------------------------
	0. User-defined case

	
	Intuition for this progression: 
		--> We start with deterministic income and an infinite life 
			because these match 'Eat the Pie' problems with closed-form
			analytic solutions
		--> Stochastic income is then added  to generate 
			precautionary saving behavior
		--> Finite life introduced for lifecycle-based consumption decisions
		--> Finally, binding liquidity constraint to generate full 
			buffer-stock model
%}

step = 1;

switch step
    case 1
        yearsWork = 51;
        yearsRet = 100;
        incomeCase = 1;
        delta = 1/1.05;			%R = 1.05 (defined below)
		%borrowingLimit = (set below to not bind);
        xmax = 50;              %set so irrelevant
        plotType = 0;

    case 2
        yearsWork = 51;
        yearsRet = 100;
        incomeCase = 1;
        delta = 0.9;
		%borrowingLimit = (set below to not bind);
        xmax = 5;
        plotType = 0;
        
    case 3
        yearsWork = 251;
        yearsRet = 0;
        incomeCase = 1;
        delta = 0.9;
		%borrowingLimit = (set below to not bind);
        xmax = 5;
        plotType = 0;        
        
    case 4
        yearsWork = 251;
        yearsRet = 0;
        incomeCase = 2;
        delta = 0.9;
		%borrowingLimit = (set below to not bind);
        xmax = 5;
        plotType = 0;       

    case 5
        yearsWork = 51;
        yearsRet = 20;
        incomeCase = 2;
        delta = 0.9;
		%borrowingLimit = (set below to not bind);
        xmax = 10;
        plotType = 0;     
    
     case 6
        yearsWork = 51;
        yearsRet = 20;
        incomeCase = 2;
        delta = 0.9;
        borrowingLimit = -.5; %Now we have a binding borrowing limit
        xmax = 10;
        plotType = 0;

	case 0
		%User-defined case
		%	NOTE: the user-defined case allows for quasi-hyperbolic discounting
		%	through the terms beta and beta_hat
        yearsWork = 100;
        yearsRet = 0;
        incomeCase = 2;
        delta = 0.9;
        beta = .7
		%beta_hat = .6
		%borrowingLimit = ;
        xmax = 10;
        plotType = 0;      
end





%%-------------------------------------------------------------------------
%%-------------------------------------------------------------------------
%% DEFINE MODEL PARAMETERS
%
%		Note: Code below uses structs to group variables. Not necessary for this code,
%		but will hopefully make extensions to this code slightly easier
%
%		Note: Commented-out variables below are defined as part of the above cases.
%		Included for conceptual clarity
%

    %- - - - - - - - - - - - - - - - 
    %Define Model Framework
    pop = 5000;                     %Size of simulated population

    %yearsWork = ;                 % # years agent works
    %yearsRet = ;                  % # years agent lives during retirement
	yearsAlive = yearsWork + yearsRet;
	
	%As a default, set eolRepay = 1  (eol = End Of Life)
	%	eolRepay = 1 is No-Ponzi Condition. Agents must die with non-negative assets
	if exist('eolRepay', 'var') ~= 1
		eolRepay = 1;
	else
		eolRepay %Already defined
	end
	
	lifecycleStruct = struct('yearsWork', yearsWork, 'yearsRet', yearsRet, 'yearsAlive', yearsAlive, 'eolRepay', eolRepay);
    %- - - - - - - - - - - - - - - - 
    
    %- - - - - - - - - - - - - - - - 
    %Define Case Specific Income Process
    %    1. Deterministic y = 1 during work; y = 0 during retirement
    %    2. E[y] = 1 during work with f(y); y = 0 during retirement
    %incomeCase = ;
	incomeStruct = buildIncome(incomeCase);
		incomeWorkMin = incomeStruct.incomeWorkMin;
		incomeWorkMax = incomeStruct.incomeWorkMax;
		incomeWorkRange = incomeStruct.incomeWorkRange;
    %- - - - - - - - - - - - - - - - 
    
    %- - - - - - - - - - - - - - - - 
    %Euler Equation Parameters
    R = 1.05;                       % Interest Rate on X
    %delta = ;                  	% Discount Factor
	if exist('beta', 'var') ~= 1
		%If beta not defined, default to exponential discounting
		beta = 1;
	else
		beta %Already defined
	end
	if exist('beta_hat', 'var') ~= 1
		%If beta-hat (naivete) not defined, default to sophisticated
		beta_hat = beta;
	else
		beta_hat %Already defined
	end
	
    util = @(x) log(x);             % Utility Fxn -- Log utility
	
	returnStruct = struct('R', R, 'delta', delta, 'beta', beta, 'beta_hat', beta_hat, 'util', util);
    %- - - - - - - - - - - - - - - - 
    
    %- - - - - - - - - - - - - - - - 
    %Savings Account X Parameters
	
	%As a default, set the borrowing limit so that it doesn't bind
	%	i.e., set the borrowing limit low enough that agents will never choose to hit it
	%	How can we do this? Set borrowingLimit = -1 * (incomeWorkMin / (R-1)) - incomWorkMin.
	%	Why? If the agent earns incomWorkMin in period t+1, then his income is only enough to 
	%		pay interest on debt, leaving the principal unchanged. 
	%		If the agent lives for a finite number of years, there is a non-zero probability she
	%		will earn incomeWorkMin in ALL remaining periods, and therefore must consume a non-positive
	%		amount in order to repay debts. However, the Inada condition prevents the agent from
	%		ever choosing this path. Hence, the borrowing limit does not bind.
	
	if exist('borrowingLimit', 'var') ~= 1
		borrowingLimit = round(-1 * (incomeWorkMin / (R-1)) - incomeWorkMin, 6);
	else
		borrowingLimit %Already defined
	end
	
    %xmax = ;                      % Maximum in Savings Acct
    
	xjump = 0.025;                   % xjump discretizes the state space
		if step >= 5
			xjump = .005;
        end
    
	Xstruct = buildX_(borrowingLimit, xmax, xjump, incomeWorkMin, incomeWorkMax);
		X_ =  Xstruct.X_;
		xlen = Xstruct.xlen;
		xzeroIndex =  Xstruct.xzeroIndex;	
    %- - - - - - - - - - - - - - - - 
%%-------------------------------------------------------------------------
%%-------------------------------------------------------------------------





%%-------------------------------------------------------------------------
%%%BACKWARD INDUCTION

    %Call script to run backward induction
	[EV__,  Ix__, V__, W__, C__, Edelta_c__, Evar_delta_c__] = ...
		backwardInduction_general(lifecycleStruct, returnStruct, Xstruct, incomeStruct);

    %Call script to simulate agent decisions given the policy functions
    %that were built in backwardInduction.m
	[simY__,  realY__, simX__, realX__, simSavings__, realSavings__, realC__] = ...
		simulateDecisions(C__, Ix__, pop, lifecycleStruct, returnStruct, Xstruct, incomeStruct);
%%-------------------------------------------------------------------------





%%-------------------------------------------------------------------------
%%Plot Consumer Behavior
%%		Below are some sample plots of consumer behavior, for reference.
%%		These plots should be updated by the user as desired
    
    %plotType = ;
    
    %- - - - - - - - - - - - - - - -
    switch plotType
        %- - - - - - - - - - - - - - - -
        %% Infinite Horizon Plots 
		%% --> these show convergence of policy and value functions
        case 1.1
            %Consumption Policy Function
            years_ = [1:yearsAlive];
            h = surf(years_(1:yearsWork), X_, C__(:, 1:yearsWork));
            xlabel('Years')
            ylabel('X_t')
            zlabel('C_t')
            set(h,'LineStyle','None')
			view([0 90])
			colormap jet
		case 1.11
            %Consumption Policy Function (at x = 0)
            years_ = [1:yearsAlive];
            plot(years_, C__(xzeroIndex, :), 'blao')
            legend('C_t (given X_t = 0)', 'Location', 'SW')
            xlabel('Years')
            grid on
        case 1.2
            %Value Function
            years_ = [1:yearsAlive];
            h = surf(years_(1:yearsWork), X_, V__(:, 1:yearsWork));
            xlabel('Years')
            ylabel('X_t')
            zlabel('V_t')
            set(h,'LineStyle','none')
			view([0 90])
			colormap jet
		case 1.21
            %Value Function (at x = 0)
            years_ = [1:yearsAlive];
            plot(years_, V__(xzeroIndex, :), 'blao')
            legend('V_t (given X_t = 0)', 'Location', 'SW')
            xlabel('Years')
            grid on				
        %- - - - - - - - - - - - - - - -
        
        %- - - - - - - - - - - - - - - -
        %% Finite Horizon Plots
        case 2.1
            %Lifecycle Savings and Consumption Profiles
            age_ = [20:20+yearsAlive-1];
            plot(age_, mean(realX__), 'r+', age_, mean(realSavings__), 'bo', age_, mean(realC__), 'g*')
            legend('X', 'Savings', 'C', 'Location', 'NW')
				if step == 5
					legend('X', 'Savings', 'C', 'Location', 'SE')
				end
			xlabel('Age')
            grid on
        
        case 2.2
            %Lifecycle Savings and Consumption for a Single Person
            person = 50;
                assert(0 < person < pop);
                
            age_ = [20:20+yearsAlive-1];
            plot(age_, mean(realSavings__), ':', age_, realSavings__(person, :), 'bo', age_, realC__(person, :), 'g*', age_, realY__(person, :), 'blas')
            legend('Average Saving of Population', 'Saving', 'C', 'Y', 'Location', 'NW')
			if step == 5
					legend('Average Saving of Population', 'Saving', 'C', 'Y', 'Location', 'SE')
				end
            xlabel('Age')
            grid on
            
        case 2.3
            %Fraction Borrowing
            age_ = [20:20+yearsAlive-1];
            plot(age_, mean(realSavings__ < 0), 'b+')
            legend('% Borrowing')
            xlabel('Age')
			axis([age_(1) age_(end) 0 1])
            grid on
            
        case 2.4
            %Consumption Policy Function
            age_ = [20:20+yearsAlive-1];
            h = surf(age_(1:yearsWork), X_, C__(:, 1:yearsWork));
            xlabel('Age')
            ylabel('X_t')
            zlabel('C_t')
            set(h,'LineStyle','none')
			colormap jet
			
		case 2.41
			%Consumption Policy at Select Ages
			years_to_display = [1:10:yearsAlive];
			for i = 1:length(years_to_display);
				colorscale = i / length(years_to_display);
				if years_to_display(i) > yearsWork;
					plot(X_, C__(:, years_to_display(i)), 'Color', 'Black')
				else
					plot(X_, C__(:, years_to_display(i)), 'Color',[0, 1-colorscale, .7])
				end
				hold on
			end
			hold off
			title('Consumption Policy Functions')
			xlabel('X_t')
            ylabel('C_t')
       
        case 2.5
            %Value Function
            age_ = [20:20+yearsAlive-1];
            h = surf(age_(1:yearsWork), X_, V__(:, 1:yearsWork));
            xlabel('Age')
            ylabel('X_t')
            zlabel('V_t')
            set(h,'LineStyle','none')
			colormap jet
		

		otherwise
		%do nothing
    end
    %- - - - - - - - - - - - - - - -
%%-------------------------------------------------------------------------



%%-------------------------------------------------------------------------
%%Check that borrowingLimit/xmax were set large enough
if (max(max(realX__)) >= 0.85 * xmax)
    display('ERROR -- Agents potentially bounded by X account maximum. Re-run with a higher xmax')
end
if (min(min(realSavings__)) <= 0.85 * borrowingLimit & borrowingLimit ~= round(-1 * (incomeWorkMin / (R-1)) - incomeWorkMin, 6) & step ~= 6)
    display('ERROR -- Agents potentially bounded by X account minimum. Re-run with a lower borrowing limit unless constraint desired')
end
%%-------------------------------------------------------------------------


